use maud::{Markup, Render, html};

use super::UIWidget;

#[allow(non_snake_case)]
pub fn Div() -> DivWidget {
    DivWidget(Vec::new(), false)
}

pub struct DivWidget(Vec<Box<dyn UIWidget>>, bool);

impl DivWidget {
    pub fn add<T: UIWidget + 'static>(mut self, element: T) -> Self {
        self.0.push(Box::new(element));
        self
    }

    pub fn add_some<T: UIWidget + 'static, X, U: Fn(&X) -> T>(
        mut self,
        option: Option<&X>,
        then: U,
    ) -> Self {
        if let Some(val) = option {
            self.0.push(Box::new(then(val)));
        }
        self
    }

    pub fn vanish(mut self) -> Self {
        self.1 = true;
        self
    }
}

impl Render for DivWidget {
    fn render(&self) -> Markup {
        self.render_with_class("")
    }
}

impl UIWidget for DivWidget {
    fn can_inherit(&self) -> bool {
        false
    }

    fn render_with_class(&self, _: &str) -> Markup {
        if self.1 {
            html! {
                @for e in &self.0 {
                    (e.as_ref())
                }
            }
        } else {
            html! {
                div {
                    @for e in &self.0 {
                        (e.as_ref())
                    }
                }
            }
        }
    }
}
