use maud::{Markup, Render, html};

use super::UIWidget;

#[allow(non_snake_case)]
pub fn Rounded<T: UIWidget + 'static>(inner: T) -> RoundedWidget {
    RoundedWidget(Box::new(inner), "full".to_owned())
}

#[allow(non_snake_case)]
pub fn RoundedMedium<T: UIWidget + 'static>(inner: T) -> RoundedWidget {
    RoundedWidget(Box::new(inner), "md".to_owned())
}

pub struct RoundedWidget(Box<dyn UIWidget>, String);

impl Render for RoundedWidget {
    fn render(&self) -> Markup {
        self.render_with_class("")
    }
}

impl UIWidget for RoundedWidget {
    fn can_inherit(&self) -> bool {
        true
    }

    fn render_with_class(&self, class: &str) -> Markup {
        if self.0.as_ref().can_inherit() {
            self.0
                .as_ref()
                .render_with_class(&format!("rounded-{} {class}", self.1))
        } else {
            html! {
                div class=(format!("rounded-{} {class}", self.1)) {
                    (self.0.as_ref())
                }
            }
        }
    }
}
