use maud::{Markup, Render, html};

use super::UIWidget;

pub struct Shadow(Box<dyn UIWidget>, String);

impl Shadow {
    pub fn medium<T: UIWidget + 'static>(inner: T) -> Shadow {
        Shadow(Box::new(inner), "md".to_owned())
    }
}

impl Render for Shadow {
    fn render(&self) -> Markup {
        self.render_with_class("")
    }
}

impl UIWidget for Shadow {
    fn can_inherit(&self) -> bool {
        true
    }

    fn render_with_class(&self, class: &str) -> Markup {
        if self.0.as_ref().can_inherit() {
            self.0
                .as_ref()
                .render_with_class(&format!("shadow-{} {class}", self.1))
        } else {
            html! {
                div class=(format!("shadow-{} {class}", self.1)) {
                    (self.0.as_ref())
                }
            }
        }
    }
}
