use maud::{Markup, Render, html};

use super::UIWidget;

#[allow(non_snake_case)]
pub fn FitWidth<T: UIWidget + 'static>(inner: T) -> FitWidthWidget {
    FitWidthWidget(Box::new(inner))
}

pub struct FitWidthWidget(Box<dyn UIWidget>);

impl Render for FitWidthWidget {
    fn render(&self) -> Markup {
        self.render_with_class("")
    }
}

impl UIWidget for FitWidthWidget {
    fn can_inherit(&self) -> bool {
        true
    }

    fn render_with_class(&self, class: &str) -> Markup {
        if self.0.as_ref().can_inherit() {
            self.0
                .as_ref()
                .render_with_class(&format!("max-w-fit {class}"))
        } else {
            html! {
                div class=(format!("max-w-fit {class}")) {
                    (self.0.as_ref())
                }
            }
        }
    }
}
